"use strict";
/**
 * @file This plugin is used to load the mobileNet for image classification. The input layer is modified to fit into any shape of input.
 * The final layer is changed to a softmax layer to match the output shape
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pipcook_core_1 = require("@pipcook/pipcook-core");
const assert = __importStar(require("assert"));
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const boa = require('@pipcook/boa');
const tf = boa.import('tensorflow');
const { Adam } = boa.import('tensorflow.keras.optimizers');
const { MobileNetV2 } = boa.import('tensorflow.keras.applications');
const { GlobalAveragePooling2D, Dropout, Dense } = boa.import('tensorflow.keras.layers');
const { Model } = boa.import('tensorflow.keras.models');
const MODEL_WEIGHTS_NAME = 'mobilenet_v2_weights_tf_dim_ordering_tf_kernels_1.0_224_no_top.h5';
const MODEL_URL = `http://ai-sample.oss-cn-hangzhou.aliyuncs.com/pipcook/models/mobilenet_python/${MODEL_WEIGHTS_NAME}`;
const MODEL_PATH = path.join(pipcook_core_1.constants.KERAS_DIR, 'models', MODEL_WEIGHTS_NAME);
/** @ignore
 * assertion test
 * @param data
 */
const assertionTest = (data) => {
    assert.ok(data.metadata.feature, 'Image feature is missing');
    assert.ok(data.metadata.feature.shape.length === 3, 'The size of an image must be 3d');
};
/**
 *  main function of the operator: load the mobilenet model
 * @param data sample data
 */
const mobilenetDefine = (data, args) => __awaiter(void 0, void 0, void 0, function* () {
    let { loss = 'categorical_crossentropy', metrics = ['accuracy'], learningRate = 0.001, decay = 0.05, recoverPath, labelMap, freeze = false } = args;
    let inputShape;
    let outputShape;
    if (!recoverPath) {
        yield data.trainLoader.next();
        assertionTest(data);
        inputShape = data.metadata.feature.shape;
        outputShape = Object.keys(data.metadata.labelMap).length;
        labelMap = data.metadata.labelMap;
    }
    else {
        const logContent = yield fs.readFile(path.join(recoverPath, '..', 'metadata.json'), 'utf8');
        const log = JSON.parse(logContent);
        labelMap = JSON.parse(log.output.dataset).labelMap;
        outputShape = Object.keys(labelMap).length;
    }
    let model;
    if (!(yield fs.pathExists(MODEL_PATH))) {
        yield pipcook_core_1.download(MODEL_URL, MODEL_PATH);
    }
    if (recoverPath) {
        model = MobileNetV2(boa.kwargs({
            include_top: false,
            weights: 'none',
            input_shape: inputShape
        }));
    }
    else {
        model = MobileNetV2(boa.kwargs({
            include_top: false,
            weights: 'imagenet',
            input_shape: inputShape
        }));
    }
    let output = model.output;
    output = GlobalAveragePooling2D()(output);
    output = Dense(1024, boa.kwargs({
        activation: 'relu'
    }))(output);
    output = Dropout(0.5)(output);
    const outputs = Dense(outputShape, boa.kwargs({
        activation: 'softmax'
    }))(output);
    model = Model(boa.kwargs({
        inputs: model.input,
        outputs: outputs
    }));
    if (freeze) {
        for (let layer of model.layers.slice(0, -10)) {
            layer.trainable = false;
        }
    }
    if (recoverPath) {
        model.load_weights(path.join(recoverPath, 'weights.h5'));
    }
    model.compile(boa.kwargs({
        optimizer: Adam(boa.kwargs({
            lr: learningRate,
            decay
        })),
        loss: loss,
        metrics: metrics
    }));
    const result = {
        model,
        metrics: metrics,
        predict: function (inputData) {
            return __awaiter(this, void 0, void 0, function* () {
                let image = tf.io.read_file(inputData.data);
                image = tf.image.decode_jpeg(image, boa.kwargs({
                    channels: 3
                }));
                const shape = tf.shape(image).numpy();
                return this.model.predict(tf.reshape(image, [1].concat(...shape.slice(0, 3)))).toString();
            });
        }
    };
    return result;
});
exports.default = mobilenetDefine;
//# sourceMappingURL=index.js.map